@@ -95,9 +95,9 @@ _SIGNATURE_RE = re.compile(
     # The function name
     r'(?P<func>\w+)' +
     # The parameters
-    r'\(((?P<self>(?:self|cls|mcs)),?)?(?P<params>.*)\)' +
+    r'\(((?P<self>(?:self|cls|mcs)),?)?(?P<params>.*?)\)' +
     # The return value (optional)
-    r'(\s*(->)\s*(?P<return>\S.*?))?'+
+    r'(\s*(->)\s*(?P<return>.*))?'+
     # The end marker
     r'\s*(\n|\s+(--|<=+>)\s+|$|\.\s+|\.\n)')
 
