@@ -1,19 +1,19 @@
-/* 
- *  You can use safely use mpi4py between multiple 
+/*
+ *  You can use safely use mpi4py between multiple
  *  Py_Initialize()/Py_Finalize() calls ...
  *  but do not blame me for the memory leaks ;-)
  *
  */
 
-#include <mpi.h>
 #include <Python.h>
+#include <mpi.h>
 
 const char helloworld[] = \
-  "from mpi4py import MPI				 \n"
+  "from mpi4py import MPI                                \n"
   "hwmess = 'Hello, World! I am process %d of %d on %s.' \n"
-  "myrank = MPI.COMM_WORLD.Get_rank()			 \n"
-  "nprocs = MPI.COMM_WORLD.Get_size()			 \n"
-  "procnm = MPI.Get_processor_name()			 \n"
+  "myrank = MPI.COMM_WORLD.Get_rank()                    \n"
+  "nprocs = MPI.COMM_WORLD.Get_size()                    \n"
+  "procnm = MPI.Get_processor_name()                     \n"
   "print (hwmess % (myrank, nprocs, procnm))             \n"
   "";
 
@@ -31,12 +31,12 @@ int main(int argc, char *argv[])
 
   Py_Initialize();
   PyRun_SimpleString(helloworld);
-  MPI_Finalize();
-  Py_Finalize();
+  MPI_Finalize(); /* MPI should be finalized */
+  Py_Finalize();  /* after finalizing Python */
 
   Py_Initialize();
   PyRun_SimpleString("from mpi4py import MPI\n");
   Py_Finalize();
-  
+
   return 0;
 }
