@@ -308,7 +308,7 @@ class TestDatatype(unittest.TestCase):
     @unittest.skipMPI('openmpi(<3.0.0)')
     @unittest.skipMPI('msmpi')
     @unittest.skipMPI('SpectrumMPI')
-    def testCreatef90ComplexSingle(self):
+    def testCreateF90ComplexSingle(self):
         (p, r) = (6, 30)
         factory = MPI.Datatype.Create_f90_complex
         args = (p, r)
@@ -327,7 +327,8 @@ class TestDatatype(unittest.TestCase):
     match_size_real    = [4, 8]
     match_size_complex = [8, 16]
     @unittest.skipMPI('MPI(<2.0)')
-    @unittest.skipMPI('openmpi', MPI.CHARACTER.Get_size() == 0)
+    @unittest.skipMPI('openmpi', (MPI.CHARACTER == MPI.DATATYPE_NULL or
+                                  MPI.CHARACTER.Get_size() == 0))
     def testMatchSize(self):
         typeclass = MPI.TYPECLASS_INTEGER
         for size in self.match_size_integer:
@@ -377,10 +378,11 @@ elif name == 'MPICH1':
 elif MPI.Get_version() < (2,0):
     combiner_map = None
 if name == 'Open MPI':
-    if MPI.CHARACTER.Get_size() == 0:
-        for dtype in datatypes_f + datatypes_f90:
-            if dtype and dtype in datatypes:
-                datatypes.remove(dtype)
+    for t in datatypes_f + datatypes_f90:
+        if t != MPI.DATATYPE_NULL:
+            if t.Get_size() == 0:
+                if t in datatypes:
+                    datatypes.remove(t)
     if (1,6,0) < version < (1,7,0):
         TestDatatype.match_size_complex[:] = []
     if version < (1,5,2):
