@@ -1,6 +1,7 @@
 from mpi4py import MPI
 import mpiunittest as unittest
-import os, tempfile
+import sys, os, tempfile
+
 
 class BaseTestFile(object):
 
@@ -30,13 +31,9 @@ class BaseTestFile(object):
         if not (amode & MPI.MODE_DELETE_ON_CLOSE):
             MPI.File.Delete(self.fname, MPI.INFO_NULL)
 
+    @unittest.skipMPI('openmpi(==2.0.0)')
+    @unittest.skipMPI('MPICH2(<1.1.0)')
     def testPreallocate(self):
-        ## MPICH2 1.0.x emits a nesting level warning
-        ## when preallocating zero size.
-        name, ver = MPI.get_vendor()
-        if not (name == 'MPICH2' and
-                ver  < (1, 1, 0)):
-            self.FILE.Preallocate(0)
         size = self.FILE.Get_size()
         self.assertEqual(size, 0)
         self.FILE.Preallocate(1)
@@ -77,6 +74,11 @@ class BaseTestFile(object):
         self.assertEqual(self.FILE.amode, self.amode)
 
     def testGetSetInfo(self):
+        #info = MPI.INFO_NULL
+        #self.FILE.Set_info(info)
+        info = MPI.Info.Create()
+        self.FILE.Set_info(info)
+        info.Free()
         info = self.FILE.Get_info()
         self.FILE.Set_info(info)
         info.Free()
@@ -94,14 +96,18 @@ class BaseTestFile(object):
                     self.FILE.Set_view(disp, etype, ftype,
                                        datarep, MPI.INFO_NULL)
                     of, et, ft, dr = self.FILE.Get_view()
-                    self.assertEqual(disp,    of)
-                    self.assertEqual(etype,   et)
-                    self.assertEqual(ftype,   ft)
+                    self.assertEqual(disp, of)
+                    self.assertEqual(etype.Get_extent(), et.Get_extent())
+                    self.assertEqual(ftype.Get_extent(), ft.Get_extent())
                     self.assertEqual(datarep, dr)
-                    #try: et.Free()
-                    #except MPI.Exception: pass
-                    #try: ft.Free()
-                    #except MPI.Exception: pass
+                    try:
+                        if not et.is_predefined: et.Free()
+                    except NotImplementedError:
+                        if et != etype: et.Free()
+                    try:
+                        if not ft.is_predefined: ft.Free()
+                    except NotImplementedError:
+                        if ft != ftype: ft.Free()
 
     def testGetSetAtomicity(self):
         atom = self.FILE.Get_atomicity()
@@ -130,6 +136,7 @@ class BaseTestFile(object):
         pos = self.FILE.Get_position_shared()
         self.assertEqual(pos, offset)
 
+    @unittest.skipMPI('openmpi(==2.0.0)')
     def testGetByteOffset(self):
         for offset in range(10):
             disp = self.FILE.Get_byte_offset(offset)
@@ -172,37 +179,18 @@ class TestFileSelf(BaseTestFile, unittest.TestCase):
     prefix = BaseTestFile.prefix + ('-%d' % MPI.COMM_WORLD.Get_rank())
 
 
-import sys
-_name, _version = MPI.get_vendor()
-if _name == 'Open MPI':
-    if _version <= (1, 2, 8):
-        if MPI.Query_thread() > MPI.THREAD_SINGLE:
-            del BaseTestFile.testPreallocate
-            del BaseTestFile.testGetSetInfo
-            del BaseTestFile.testGetSetAtomicity
-            del BaseTestFile.testSync
-            del BaseTestFile.testGetAmode
-            del BaseTestFile.testGetSetSize
-            del BaseTestFile.testGetSetView
-            del BaseTestFile.testGetByteOffset
-            del BaseTestFile.testGetTypeExtent
-            del BaseTestFile.testSeekGetPosition
-            del BaseTestFile.testSeekGetPositionShared
-    if 'win' in sys.platform:
-        del BaseTestFile
-        del TestFileNull
-        del TestFileSelf
-else:
-    try:
-        dummy = BaseTestFile()
-        dummy.COMM = MPI.COMM_SELF
-        dummy.setUp()
-        dummy.tearDown()
-        del dummy
-    except NotImplementedError:
-        del BaseTestFile
-        del TestFileNull
-        del TestFileSelf
+def have_feature():
+    case = BaseTestFile()
+    case.COMM = TestFileSelf.COMM
+    case.prefix = TestFileSelf.prefix
+    case.setUp()
+    case.tearDown()
+try:
+    have_feature()
+except NotImplementedError:
+    unittest.disable(BaseTestFile, 'mpi-file')
+    unittest.disable(TestFileNull, 'mpi-file')
+
 
 if __name__ == '__main__':
     unittest.main()
