@@ -1127,6 +1127,23 @@ class TestMessageVectorW(unittest.TestCase):
         MPI.Free_mem(sbuf)
         MPI.Free_mem(rbuf)
 
+    @unittest.skipIf(pypy_lt_53, 'pypy(<5.3)')
+    def testMessageBottom(self):
+        sbuf = b"abcxyz"
+        rbuf = bytearray(6)
+        saddr = MPI.Get_address(sbuf)
+        raddr = MPI.Get_address(rbuf)
+        stype = MPI.Datatype.Create_struct([6], [saddr], [MPI.CHAR]).Commit()
+        rtype = MPI.Datatype.Create_struct([6], [raddr], [MPI.CHAR]).Commit()
+        smsg = [MPI.BOTTOM,  [1], [0] , [stype]]
+        rmsg = [MPI.BOTTOM, ([1], [0]), [rtype]]
+        try:
+            Alltoallw(smsg, rmsg)
+            self.assertEqual(sbuf, rbuf)
+        finally:
+            stype.Free()
+            rtype.Free()
+
     @unittest.skipIf(pypy_lt_53, 'pypy(<5.3)')
     def testMessageBytes(self):
         sbuf = b"abc"
