@@ -70,7 +70,9 @@ class BaseTestP2PObj(object):
             self.assertTrue(req)
             rmess = self.COMM.recv(buf, rank, 0)
             self.assertTrue(req)
-            flag = req.Test()
+            flag = False
+            while not flag:
+                flag = req.Test()
             self.assertTrue(flag)
             self.assertFalse(req)
             self.assertEqual(rmess, smess)
@@ -366,7 +368,9 @@ class BaseTestP2PObj(object):
             self.assertTrue(req)
             rmess = self.COMM.recv(None, rank, 0)
             self.assertTrue(req)
-            flag = req.Test()
+            flag = False
+            while not flag:
+                flag = req.Test()
             self.assertTrue(flag)
             self.assertFalse(req)
             self.assertEqual(rmess, smess)
@@ -380,6 +384,30 @@ class BaseTestP2PObj(object):
             self.assertFalse(req)
             self.assertEqual(rmess, smess)
 
+    def testCancel(self):
+        size = self.COMM.Get_size()
+        rank = self.COMM.Get_rank()
+        status = MPI.Status()
+        for smess in messages:
+            req = self.COMM.issend(smess, rank)
+            self.assertTrue(req)
+            req.cancel()
+            flag = req.get_status(status)
+            cancelled = status.Is_cancelled()
+            self.assertTrue(req)
+            if cancelled:
+                self.assertTrue(flag)
+                req.Free()
+                self.assertFalse(req)
+            else:
+                self.assertFalse(flag)
+                rmess = self.COMM.recv(None,  rank, 0)
+                flag = req.get_status()
+                self.assertTrue(flag)
+                flag, _ = req.test()
+                self.assertTrue(flag)
+                self.assertEqual(rmess, smess)
+
     def testIRecvAndBSend(self):
         comm = self.COMM
         rank = comm.Get_rank()
@@ -542,10 +570,9 @@ class BaseTestP2PObj(object):
             for smess in messages:
                 request = comm.issend(smess, comm.rank, 123)
                 self.assertTrue(request)
-                flag = comm.iprobe(MPI.ANY_SOURCE, MPI.ANY_TAG, status)
+                while not comm.iprobe(MPI.ANY_SOURCE, MPI.ANY_TAG, status): pass
                 self.assertEqual(status.source, comm.rank)
                 self.assertEqual(status.tag, 123)
-                self.assertTrue(flag)
                 comm.probe(MPI.ANY_SOURCE, MPI.ANY_TAG, status)
                 self.assertEqual(status.source, comm.rank)
                 self.assertEqual(status.tag, 123)
@@ -557,13 +584,110 @@ class BaseTestP2PObj(object):
                 obj = comm.recv(None, comm.rank, 123)
                 self.assertEqual(obj, smess)
                 self.assertTrue(request)
-                flag, obj = request.test()
+                obj = request.wait()
                 self.assertFalse(request)
-                self.assertTrue(flag)
                 self.assertEqual(obj, None)
         finally:
             comm.Free()
 
+    def testWaitSomeRecv(self):
+        comm = self.COMM.Dup()
+        rank = comm.Get_rank()
+        reqs = [comm.irecv(source=rank, tag=i) for i in range(6)]
+        for indexlist in ([5], [3,1,2], [0,4]):
+            for i in indexlist:
+                comm.ssend("abc", dest=rank, tag=i)
+            statuses = []
+            idxs, objs = MPI.Request.waitsome(reqs, statuses)
+            self.assertEqual(sorted(idxs), sorted(indexlist))
+            self.assertEqual(objs, ["abc"]*len(idxs))
+            self.assertFalse(any(reqs[i] for i in idxs))
+            self.assertEqual(len(statuses), len(idxs))
+            self.assertTrue(all(s.source == rank for s in statuses))
+            self.assertTrue(all(s.tag in indexlist for s in statuses))
+            self.assertTrue(all(s.error == MPI.SUCCESS for s in statuses))
+        idxs, objs = MPI.Request.waitsome(reqs)
+        self.assertEqual(idxs, None)
+        self.assertEqual(objs, None)
+        self.assertFalse(any(reqs))
+        comm.Free()
+
+    def testTestSomeRecv(self):
+        comm = self.COMM.Dup()
+        rank = comm.Get_rank()
+        reqs = [comm.irecv(source=rank, tag=i) for i in range(6)]
+        statuses = []
+        idxs, objs = MPI.Request.testsome(reqs, statuses)
+        self.assertEqual(idxs, [])
+        self.assertEqual(objs, [])
+        self.assertTrue(all(reqs))
+        self.assertEqual(statuses, [])
+        for indexlist in ([5], [], [3,1,2], [], [0,4]):
+            for i in indexlist:
+                comm.ssend("abc", dest=rank, tag=i)
+            statuses = []
+            idxs, objs = MPI.Request.testsome(reqs, statuses)
+            self.assertEqual(sorted(idxs), sorted(indexlist))
+            self.assertEqual(objs, ["abc"]*len(idxs))
+            self.assertFalse(any(reqs[i] for i in idxs))
+            self.assertEqual(len(statuses), len(idxs))
+            self.assertTrue(all(s.source == rank for s in statuses))
+            self.assertTrue(all(s.tag in indexlist for s in statuses))
+            self.assertTrue(all(s.error == MPI.SUCCESS for s in statuses))
+        idxs, objs = MPI.Request.testsome(reqs)
+        self.assertEqual(idxs, None)
+        self.assertEqual(objs, None)
+        self.assertFalse(any(reqs))
+        comm.Free()
+
+    def testWaitSomeSend(self):
+        comm = self.COMM.Dup()
+        rank = comm.Get_rank()
+        reqs = [comm.issend("abc", dest=rank, tag=i) for i in range(6)]
+        for indexlist in ([5], [3,1,2], [0,4]):
+            for i in indexlist:
+                msg = comm.recv(source=rank, tag=i)
+                self.assertEqual(msg, "abc")
+            idxs, objs = MPI.Request.waitsome(reqs)
+            while sorted(idxs) != sorted(indexlist):
+                i, o = MPI.Request.waitsome(reqs)
+                idxs.extend(i)
+                objs.extend(o)
+            self.assertEqual(sorted(idxs), sorted(indexlist))
+            self.assertEqual(objs, [None]*len(idxs))
+            self.assertFalse(any(reqs[i] for i in idxs))
+        idxs, objs = MPI.Request.waitsome(reqs)
+        self.assertEqual(idxs, None)
+        self.assertEqual(objs, None)
+        self.assertFalse(any(reqs))
+        comm.Free()
+
+    def testTestSomeSend(self):
+        comm = self.COMM.Dup()
+        rank = comm.Get_rank()
+        reqs = [comm.issend("abc", dest=rank, tag=i) for i in range(6)]
+        idxs, objs = MPI.Request.testsome(reqs)
+        self.assertEqual(idxs, [])
+        self.assertEqual(objs, [])
+        self.assertTrue(all(reqs))
+        for indexlist in ([5], [], [3,1,2], [], [0,4]):
+            for i in indexlist:
+                msg = comm.recv(source=rank, tag=i)
+                self.assertEqual(msg, "abc")
+            idxs, objs = MPI.Request.testsome(reqs)
+            while sorted(idxs) != sorted(indexlist):
+                i, o = MPI.Request.testsome(reqs)
+                idxs.extend(i)
+                objs.extend(o)
+            self.assertEqual(sorted(idxs), sorted(indexlist))
+            self.assertEqual(objs, [None]*len(idxs))
+            self.assertFalse(any(reqs[i] for i in idxs))
+        idxs, objs = MPI.Request.testsome(reqs)
+        self.assertEqual(idxs, None)
+        self.assertEqual(objs, None)
+        self.assertFalse(any(reqs))
+        comm.Free()
+
 
 class TestP2PObjSelf(BaseTestP2PObj, unittest.TestCase):
     COMM = MPI.COMM_SELF
